? destructuring-irc-message.lisp
? test/package.fasl
? test/test-parse-message.fasl
? test/test-protocol.fasl
Index: cl-irc.asd
===================================================================
RCS file: /project/cl-irc/cvsroot/cl-irc/cl-irc.asd,v
retrieving revision 1.3
diff -u -r1.3 cl-irc.asd
--- cl-irc.asd	24 Jan 2006 22:10:58 -0000	1.3
+++ cl-irc.asd	8 Mar 2006 18:10:17 -0000
@@ -16,7 +16,7 @@
     :version "0.5.2"
     :licence "MIT"
     :description "Common Lisp interface to the IRC protocol"
-    :depends-on (:split-sequence :trivial-sockets)
+    :depends-on (:split-sequence :trivial-sockets :flexi-streams)
     :properties ((#:author-email . "cl-irc-devel@common-lisp.net")
                  (#:date . "$Date: 2006/01/24 22:10:58 $")
                  ((#:albert #:output-dir) . "doc/api-doc/")
Index: command.lisp
===================================================================
RCS file: /project/cl-irc/cvsroot/cl-irc/command.lisp,v
retrieving revision 1.16
diff -u -r1.16 command.lisp
--- command.lisp	15 Feb 2006 20:14:21 -0000	1.16
+++ command.lisp	8 Mar 2006 18:10:17 -0000
@@ -250,8 +250,10 @@
                      (logging-stream t))
   "Connect to server and return a connection object."
   (let* ((stream (socket-connect server port))
+         (input-stream (flexi-streams:make-flexi-stream stream :element-type 'character :external-format '(:latin-1 :eol-style :crlf)))
          (connection (make-connection :connection-type connection-type
                                       :network-stream stream
+                                      :input-stream input-stream
                                       :client-stream logging-stream
                                       :server-name server))
          (user (make-user connection
Index: event.lisp
===================================================================
RCS file: /project/cl-irc/cvsroot/cl-irc/event.lisp,v
retrieving revision 1.24
diff -u -r1.24 event.lisp
--- event.lisp	22 Feb 2006 18:59:13 -0000	1.24
+++ event.lisp	8 Mar 2006 18:10:17 -0000
Index: protocol.lisp
===================================================================
RCS file: /project/cl-irc/cvsroot/cl-irc/protocol.lisp,v
retrieving revision 1.39
diff -u -r1.39 protocol.lisp
--- protocol.lisp	22 Feb 2006 18:59:13 -0000	1.39
+++ protocol.lisp	8 Mar 2006 18:10:17 -0000
@@ -117,7 +117,11 @@
    (network-stream
     :initarg :network-stream
     :accessor network-stream
-    :documentation "Stream used to talk to the IRC server.")
+    :documentation "Stream used to talk binary to the IRC server.")
+   (input-stream
+    :initarg :input-stream
+    :accessor input-stream
+    :documentation "Stream used to receive messages from the IRC server.")
    (server-capabilities
     :initform *default-isupport-values*
     :accessor server-capabilities
@@ -183,16 +187,19 @@
 (defgeneric re-apply-case-mapping (connection))
 
 (defun make-connection (&key (connection-type 'connection)
-                             (user nil)
-                             (server-name "")
-                             (network-stream nil)
-                             (client-stream t)
-                             (hooks nil))
+                        (user nil)
+                        (server-name "")
+                        (network-stream nil)
+                        (output-stream nil)
+                        (input-stream nil)
+                        (client-stream t)
+                        (hooks nil))
   (let ((connection (make-instance connection-type
-                                   :user user
-                                   :server-name server-name
-                                   :network-stream network-stream
-                                   :client-stream client-stream)))
+                       :user user
+                       :server-name server-name
+                       :network-stream network-stream
+                       :input-stream input-stream
+                       :client-stream client-stream)))
     (dolist (hook hooks)
       (add-hook connection (car hook) (cadr hook)))
     connection))
@@ -292,13 +299,31 @@
 (defun read-message-loop (connection)
   (loop while (read-message connection)))
 
+(defun try-decode-line (line external-formats)
+  (loop for external-format in external-formats
+        for decoded = nil
+        for error = nil
+        do (multiple-value-setq (decoded error)
+               (handler-case
+                   (flexi-streams:with-input-from-sequence (in line)
+                     (let ((flexi (flexi-streams:make-flexi-stream in :external-format external-format)))
+                       (read-line flexi)))
+                 (flexi-streams:flexi-stream-encoding-error ()
+                   nil)))
+        if decoded
+          do (return decoded)))
+
 (defmethod read-irc-message ((connection connection))
   "Read and parse an IRC-message from the `connection'."
   (handler-case
-    (let ((message (create-irc-message
-                    (read-line (network-stream connection) t))))
-      (setf (connection message) connection)
-      message)
+      (let* ((line (read-line (input-stream connection) t))
+             (message (create-irc-message
+                       (concatenate 'string
+                                    (try-decode-line (map 'vector #'char-code line)
+                                                     *default-incoming-external-formats*)
+                                    '(#\Return)))))
+        (setf (connection message) connection)
+        message)
     (end-of-file ())))
        ;; satisfy read-message-loop assumption of nil when no more messages
 
@@ -306,9 +331,11 @@
                              &rest arguments)
   "Turn the arguments into a valid IRC message and send it to the
 server, via the `connection'."
-  (let ((raw-message (apply #'make-irc-message command arguments)))
-    (write-sequence raw-message (network-stream connection))
-    (force-output (network-stream connection))
+  (let ((raw-message (apply #'make-irc-message command arguments))
+        (output (flexi-streams:make-flexi-stream (network-stream connection) :element-type 'character
+                                                 :external-format '(:latin-1 :eol-style :crlf))))
+    (write-sequence raw-message output)
+    (force-output output)
     raw-message))
 
 (defmethod get-hooks ((connection connection) (class symbol))
@@ -370,7 +397,7 @@
 ;;
 ;; DCC Connection
 ;;
-
+;;; big TODO: flexi-streams
 (defclass dcc-connection ()
   ((user
     :initarg :user
Index: utility.lisp
===================================================================
RCS file: /project/cl-irc/cvsroot/cl-irc/utility.lisp,v
retrieving revision 1.12
diff -u -r1.12 utility.lisp
--- utility.lisp	20 Feb 2006 20:26:54 -0000	1.12
+++ utility.lisp	8 Mar 2006 18:10:17 -0000
@@ -54,9 +54,8 @@
 parameters."
   (let ((*print-circle* nil))
     (format nil
-            "~A~{ ~A~}~@[ :~A~]~A~A"
-            command (butlast arguments) (car (last arguments))
-            #\Return #\Linefeed)))
+            "~A~{ ~A~}~@[ :~A~]~%"
+            command (butlast arguments) (car (last arguments)))))
 
 (defun make-ctcp-message (string)
   "Return a valid IRC CTCP message, as a string, composed by
@@ -104,13 +103,13 @@
 
 (defun socket-connect (server port)
   "Create a socket connected to `server':`port' and return stream for it."
-  (trivial-sockets:open-stream server port))
+  (trivial-sockets:open-stream server port :element-type '(unsigned-byte 8)))
 
 
 (defun cut-between (string start-char end-chars &key (start 0) (cut-extra t))
-  "If `start-char' is not nil, cut string between `start-char' and any
-of the `end-chars', from `start'.  If `start-char' is nil, cut from
-`start' until any of the `end-chars'.
+  "If `start-char' is not nil, cut string between `start-char'
+and any of the `end-chars', from `start'.  If `start-char' is
+nil, cut from `start' until any of the `end-chars'.
 
 If `cut-extra' is t, we will cut from start + 1 instead of just
 `start'.
Index: variable.lisp
===================================================================
RCS file: /project/cl-irc/cvsroot/cl-irc/variable.lisp,v
retrieving revision 1.7
diff -u -r1.7 variable.lisp
--- variable.lisp	15 Feb 2006 19:15:21 -0000	1.7
+++ variable.lisp	8 Mar 2006 18:10:17 -0000
@@ -41,6 +41,20 @@
     ("PREFIX" ,*default-isupport-PREFIX*)
     ("TARGMAX")))
 
+(defparameter *default-outgoing-external-format* '(:latin-1 :eol-style :lf)
+  "The external-format we use to encode outgoing messages. This
+  should be an external format spec that flexi-streams accepts,
+  with :eol-style :lf.")
+
+(defparameter *default-incoming-external-formats* '((:utf-8 :eol-style :lf)
+                                                    (:latin-1 :eol-style :lf))
+  "The external-formats we use to decode incoming messages. This
+  should be a list of external format spec that flexi-streams accepts,
+  with :eol-style :lf.
+
+The external formats are tried in order, until one decodes the
+message without encoding errors. Note that the last external
+format should be a single-byte one like latin-1.")
 
 (defvar *dcc-connections* nil)
 
